/**
 * Copyright (C) 2015 BonitaSoft S.A.
 * BonitaSoft, 32 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 **/

package org.bonitasoft.migration.versions.to_7_0_0

/**
 * @author Elias Ricken de Medeiros
 */
class GlobalApplicationsMessageBuilder {

    private List<ApplicationRetriever> retrievers
    private final TenantApplicationsMessageBuilder tenantApplicationsMessageBuilder

    GlobalApplicationsMessageBuilder(final List<ApplicationRetriever> retrievers, TenantApplicationsMessageBuilder tenantApplicationsMessageBuilder) {
        this.tenantApplicationsMessageBuilder = tenantApplicationsMessageBuilder
        this.retrievers = retrievers
    }

    def String buildMessage() {
        StringBuilder stb = new StringBuilder()
        retrievers.each { retriever ->
            List<TenantApplications> allInvalidApplications = retriever.retrieveApplications()
            if (allInvalidApplications != null && !allInvalidApplications.isEmpty()) {
                stb.append(retriever.getHeader())
                allInvalidApplications.each { tenantInvalidApplications ->
                    stb.append(tenantApplicationsMessageBuilder.buildMessage(tenantInvalidApplications))
                }
            }
        }
        return stb.toString()
    }

}
